#! /usr/bin/env python

import sys
from getopt import getopt, GetoptError

import eyeD3
from eyeD3.tag import Mp3AudioFile


def usage(msg):
    pname = sys.argv[0]
    sys.stderr.write("%s: %s\n" % (pname, msg))
    sys.stderr.write("Usage: %s file.mp3...\n" % pname)
    sys.stderr.write("Usage: %s [-t encoding|--to-encoding=encoding] file.mp3...\n" % pname)
    sys.exit(1)

try:
    optlist, args = getopt(sys.argv[1:], "t:", ["to-encoding="])
except GetoptError:
   usage("invalid option")


dst_encoding = None

for _opt, _arg in optlist:
    if _opt in ('-t', '--to-encoding'):
        dst_encoding = _arg
try:
   del _opt, _arg
except NameError:
   pass

if not args:
   usage("too few arguments")

if not dst_encoding:
    from m_lib.defenc import default_encoding
    dst_encoding = default_encoding


def id3tag_recode(value):
    try:
        return value.encode(dst_encoding)
    except UnicodeEncodeError:
        return value.encode('latin1')

def printFrame(title, frame):
    if frame:
        print '%s: %s' % (title, frame)

def printFramesList(title, frames):
    frames = [frame for frame in frames if frame]
    if frames:
        print '%s: %s' % (title, ', '.join(frames))

def printTrack(title, frame):
    if frame:
        track, total = frame
        if total and track:
            print '%s: %d (of %d)' % (title, track, total)
        elif track:
            print '%s: %d' % (title, track)
        elif total:
            print '%s: UNKNOWN of %d' % (title, total)

for filename in args:
    print "File:", filename
    mp3_file = Mp3AudioFile(filename)
    print "Bitrate:", mp3_file.getBitRateString()
    print "Duration:", mp3_file.getPlayTimeString(), 's'

    id3tag = mp3_file.getTag()
    if not id3tag:
        sys.exit()

    for version in eyeD3.ID3_V2, eyeD3.ID3_V1:
        id3tag = eyeD3.Tag()
        if not id3tag.link(filename, version):
            continue

        print '-----'
        printFrame("id3tag version", id3tag.getVersionStr())
        printFrame("Artist", id3tag_recode(id3tag.getArtist()))
        printFrame("Album", id3tag_recode(id3tag.getAlbum()))
        printFrame("Title", id3tag_recode(id3tag.getTitle()))
        dates = id3tag.getDate()
        if dates:
            printFramesList("Date", [date.getDate() for date in dates])
        try:
            printFrame("Genre", id3tag.getGenre().getName())
        except: # Unknown id3tag or no id3tag at all
            pass
        printTrack("Track num", id3tag.getTrackNum())
        printFramesList("Disc num", [str(n) for n in id3tag.getDiscNum() if n])
        printFramesList("Comments", [id3tag_recode(frame.comment) for frame in id3tag.getComments() if frame])
        printFramesList("Lyrics", [frame.description for frame in id3tag.getLyrics()])
        if id3tag.getImages():
            print "Has images"
        printFramesList("URLs", [frame.url for frame in id3tag.getURLs()])
